-------------------------------------------
---  Copyright HDRsoft - www.hdrsoft.com   
-------------------------------------------

local LrApplication = import 'LrApplication'
local LrDialogs = import 'LrDialogs'
local LrFunctionContext = import 'LrFunctionContext'
local LrErrors = import 'LrErrors'
local LrDate = import 'LrDate'
local LrFileUtils = import 'LrFileUtils'
local LrPathUtils = import 'LrPathUtils'
local LrShell = import 'LrShell'
local LrTasks = import 'LrTasks'
local LrLogger = import 'LrLogger'
local prefs = import 'LrPrefs'.prefsForPlugin() 


local myLogger = LrLogger('PhotomatixLogger')
myLogger:enable("print") 

CreateHdrUploadTask = {}

function CreateHdrUploadTask.outputToLog(message)
	myLogger:trace(message)
end

function CreateHdrUploadTask.processRenderedPhotos( functionContext, exportContext )
	local pmdebug = true
	
	local lrVersion=LrApplication.versionTable
	if lrVersion then
		prefs.LrVersion=lrVersion().major
	else
		prefs.LrVersion=1
	end
	CreateHdrUploadTask.outputToLog(string.format('CreateHdrUploadTask.processRenderedPhotos prefs.photomatix4Path: %s',prefs.photomatix4Path))

-------------------------- Showing plugin dialog and getting the options selected ------------------------------------	
	require 'CreateHdrSettings'

	CreateHdrSettings.getOptions()

 	if prefs.processType=='none' then
		return
	end
	
-------------------------- Getting path to PhotomatixClient ------------------------------------	
	local pathExist=false
	local appPath=prefs.photomatix4Path

	require 'PhotomatixApp'
	
	if appPath then
		pathExist=LrFileUtils.exists(appPath) 
		if pathExist then
			pathExist=PhotomatixApp.clientExists(appPath, true)
		else
			CreateHdrUploadTask.outputToLog(string.format('LrFileUtils.exists returned false for: %s',appPath))
		end	
	end 
	
	if not pathExist then
		CreateHdrUploadTask.outputToLog("prefs.photomatix4Path does not exist")
		PhotomatixApp.getPath()
		appPath=prefs.photomatix4Path
		if not LrFileUtils.exists(appPath) then
			CreateHdrUploadTask.outputToLog(string.format('2nd try: LrFileUtils.exists returned false for: %s',appPath))
			LrErrors.throwUserError("Could not find Photomatix Pro application compatible with this plug-in") 		
			return
		end
	end

	
	local sysTempPath = LrPathUtils.getStandardFilePath('temp')
	local tempPath = nil
	repeat
		local now = LrDate.currentTime()
		tempPath = LrPathUtils.child(sysTempPath, LrDate.timeToUserFormat(now, 'temp_lrexport2photomatix_%m%d%H%M%S'))
		if LrFileUtils.exists(tempPath) then
			tempPath = nil
		else
			LrFileUtils.createAllDirectories(tempPath)
		end
	until tempPath
	
	if pmdebug then 
		CreateHdrUploadTask.outputToLog(string.format('tempPath: %s',tempPath))
	end

	local exportSession = exportContext.exportSession
	
	local activeCatalog = LrApplication.activeCatalog()

    -- we retrieve the photo to stack with now so that LR still stacks with the originally selected photo in case the user selected another photo before Photomatix reimports back the resulting image
	local photoToStackWith=nil
	local pathPhotoToStackWith=nil
	if (prefs.stackWith) then
		if prefs.LrVersion >= 3 then
			activeCatalog:withReadAccessDo( function()
			photoToStackWith = activeCatalog:getTargetPhoto()	
			end )
			pathPhotoToStackWith=photoToStackWith:getRawMetadata('path')
		else
			activeCatalog:withReadAccessDo( function()
			photoToStackWith = activeCatalog.targetPhoto	
			pathPhotoToStackWith=photoToStackWith.path
			end )
		end
		if pmdebug then
			CreateHdrUploadTask.outputToLog(string.format('path of photo to stack with: %s', pathPhotoToStackWith))
		end
	end

	
	
	local exportParams = exportContext.propertyTable
	local nPhotos = exportSession:countRenditions()

	local progressScope = exportContext:configureProgress{
						title = LOC( "$$$/Photomatix/Upload/ProgressTransfer=Transferring ^1 photos to Photomatix Pro for HDR merge", nPhotos ),
					}

	local files = {}
	local nFiles=0
	local inputFilesArgs=""
	
-------------------------- Exporting photos to temporary directory ------------------------------------

	for i, rendition in exportContext:renditions{ stopIfCanceled = true } do
	
		local success, pathOrMessage = rendition:waitForRender()
		
		if progressScope:isCanceled() then break end
		
		if success then
			local copyPath = LrPathUtils.child(tempPath, LrPathUtils.leafName(pathOrMessage))
			LrFileUtils.copy(pathOrMessage, copyPath)
			table.insert(files, copyPath)
			inputFilesArgs=string.format('%s \"%s\"',inputFilesArgs,copyPath)
			nFiles = nFiles + 1
		else
			LrErrors.throwUserError("Lightroom rendition failed. Please check that the selected photos exist.")
			---	error(pathOrMessage)
		end 
	end

	if nFiles > 1 then
		local doesExist=LrFileUtils.exists(appPath) 

		if doesExist then
		
-------------------------- Building command line arguments to pass to PhotomatixClient ------------------------------------
		
			local PhotomatixClientCommand=string.format('\"%s\" -o \"%s\"', appPath, prefs.outputPath)
			local ldrBits=32
			if prefs.outputFormat=='tif16' then
				ldrBits=16
			end

			PhotomatixClientCommand=string.format('%s -bi %d', PhotomatixClientCommand, ldrBits)

			if prefs.chkAlign then
				if prefs.chkCrop then
					PhotomatixClientCommand=string.format('%s -a%sc', PhotomatixClientCommand, prefs.alignType)
				else
					PhotomatixClientCommand=string.format('%s -a%sn', PhotomatixClientCommand, prefs.alignType)
				end
				if prefs.chkPerspective then
					PhotomatixClientCommand=string.format('%sp', PhotomatixClientCommand)
				else
					PhotomatixClientCommand=string.format('%sn', PhotomatixClientCommand)
				end

			end

			if prefs.chkGhosting then
				if (prefs.ghostingType=='1') then
					PhotomatixClientCommand=string.format('%s -g1', PhotomatixClientCommand)
				end
				if (prefs.ghostingType=='2') then
					if (prefs.ghostingDetection=='high') then
						PhotomatixClientCommand=string.format('%s -g2h', PhotomatixClientCommand)
					else
						PhotomatixClientCommand=string.format('%s -g2n', PhotomatixClientCommand)
					end		
				end
			end

			if prefs.chkReduceNoise then
				PhotomatixClientCommand=string.format('%s -no%s', PhotomatixClientCommand, prefs.nrWhat)			
			end
			
			if prefs.waitForPhotomatix then
				PhotomatixClientCommand=string.format('%s -w', PhotomatixClientCommand)			
			end

			if MAC_ENV then	-- we add the path of the folder containing the exported files to the arguments passed to PhotomatixClient
				PhotomatixClientCommand=string.format('%s -f %s',PhotomatixClientCommand, tempPath)			
			end

			local outputTempFile=string.format('%sCmdOutput', tempPath)
			if (WIN_ENV) then
				PhotomatixClientCommand=string.format('%s %s %s > \"%s\"', PhotomatixClientCommand, prefs.processType, inputFilesArgs, outputTempFile)		
				PhotomatixClientCommand=string.format('\"%s\"', PhotomatixClientCommand)
			else
				PhotomatixClientCommand=string.format('%s %s %s', PhotomatixClientCommand, prefs.processType, inputFilesArgs)		
			end

			progressScope:done()
				
			CreateHdrUploadTask.outputToLog(string.format('PhotomatixClientCommand: %s', PhotomatixClientCommand))

			if (WIN_ENV) then 
				LrFileUtils.delete(outputTempFile) -- in case it is there from before
			end
			
-------------------------- Calling PhotomatixClient ------------------------------------

			local retour=-1

			local txtProgress=LOC "$$$/Photomatix/Progress/AlignAndMerge=Aligning and merging to HDR..."
			if prefs.chkReduceNoise then
				if prefs.chkAlign then
					txtProgress=LOC "$$$/Photomatix/Progress/AlignDenoiseAndMerge=Aligning, denoising and merging to HDR..."
				else
					txtProgress=LOC "$$$/Photomatix/Progress/DenoiseAndMerge=Denoising and merging to HDR..."
				end
			else
				if not prefs.chkAlign then
					txtProgress=LOC "$$$/Photomatix/Progress/Merge=Merging to HDR..."
				end		
			end	
			
			LrFunctionContext.callWithContext('ExecuteCreateHDR', function(context)

				local progressScope = LrDialogs.showModalProgressDialog({
					title = LOC "$$$/Photomatix/General/Create32HDR=Create 32-bit HDR",
					caption = txtProgress,
					cannotCancel = true,
					functionContext = context,
				})

				progressScope:setIndeterminate()

				retour=LrTasks.execute(PhotomatixClientCommand)
			
				progressScope:done()

		    end)
			
			---	retour=LrTasks.execute(PhotomatixClientCommand)
			
			if pmdebug then
				CreateHdrUploadTask.outputToLog(string.format('retour PhotomatixClient: %d', retour))
			end
			
			--- tempo: Test under LR3
			---if pmdebug then
			---	return 
			---end
-------------------------- Returning from PhotomatixClient ------------------------------------

			if WIN_ENV then  -- Read output - this should be only one line
				local outputMessage
				local handle = io.open(outputTempFile, 'rb')
				if handle then
					CreateHdrUploadTask.outputToLog("Handle ok")
					outputMessage = handle:read('*a')
               		   handle:close()
				end

				if outputMessage == nil then
					outputMessage = "???"
				end

				local formattedMsg = string.format(outputMessage)
				CreateHdrUploadTask.outputToLog(string.format(outputMessage))

				if string.match(formattedMsg, '^FAIL') then
					LrDialogs.message("Error while exporting: " .. string.sub(outputMessage, 7), nil, "critical")
				end
			end

			if (retour==0 and prefs.waitForPhotomatix) then
				activeCatalog = LrApplication.activeCatalog()
				local undoActionName=string.format('Add %s to catalog', LrPathUtils.leafName(prefs.outputPath))

				if LrFileUtils.exists(prefs.outputPath)	then
					CreateHdrUploadTask.outputToLog(string.format('reimporting file: %s',prefs.outputPath))
				else
					CreateHdrUploadTask.outputToLog(string.format('file to reimport does not exist: %s',prefs.outputPath))
					LrDialogs.message("Could not access file to re-import", prefs.outputPath, "info")
				end
				
				if (prefs.stackWith) then
					photoToStackWith=nil
					activeCatalog:withReadAccessDo( function()
						photoToStackWith = activeCatalog:findPhotoByPath(pathPhotoToStackWith)
					end )
				end
				
				-- See if the photo we are importing is already in the catalog
				local existingPhoto = nil
				activeCatalog:withReadAccessDo( function()
				    existingPhoto = activeCatalog:findPhotoByPath(prefs.outputPath)
				end )
				
				local status, err
				if (photoToStackWith==nil) then
					activeCatalog:withWriteAccessDo( undoActionName, function()
						if prefs.LrVersion > 2 then
							-- Work around bug in LR3 that raises an error when adding a photo that already exists
							status, err = LrTasks.pcall ( function()
								activeCatalog:addPhoto(prefs.outputPath)
							end )
						else
							status = 1
							activeCatalog:addPhoto(prefs.outputPath)
						end
					end )
				else
					if pmdebug then
						CreateHdrUploadTask.outputToLog(string.format('going to add photo and stack it  - path: %s', prefs.outputPath))
					end

					activeCatalog:withWriteAccessDo( undoActionName, function()
						if prefs.LrVersion > 2 then
							-- Work around bug in LR3 that raises an error when adding a photo that already exists
							status, err = LrTasks.pcall ( function()
								activeCatalog:addPhoto(prefs.outputPath, photoToStackWith, 'below')
							end )
						else
							status = 1
							activeCatalog:addPhoto(prefs.outputPath, photoToStackWith, 'below')
						end
					end )				
				end
				
				-- If importing into 
				if not status then
					if existingPhoto == nil then
						LrDialogs.message("Error while importing photo:",  err, "critical")
					-- else ignore error
					end
				end
			end

			if (WIN_ENV) then 
				LrFileUtils.delete(outputTempFile)
			end	
		end
	end
end

